@@ -1,16 +1,17 @@
-use {Line, LineSegment, LineEquation};
-use scalar::Scalar;
-use generic_math::{Point, Vector, Rect, rect, Transform2D};
+pub use crate::flatten_cubic::Flattened;
+use crate::{Line, LineSegment, LineEquation, QuadraticBezierSegment};
+use crate::scalar::Scalar;
+use crate::generic_math::{Point, Vector, Rect, rect, Transform2D};
+use crate::flatten_cubic::{flatten_cubic_bezier, find_cubic_bezier_inflection_points};
+use crate::cubic_to_quadratic::*;
+use crate::cubic_bezier_intersections::cubic_bezier_intersections_t;
+use crate::monotonic::Monotonic;
+use crate::utils::{min_max, cubic_polynomial_roots};
+use crate::segment::{Segment, FlattenedForEach, approximate_length_from_flattening, BoundingRect};
 use arrayvec::ArrayVec;
-use flatten_cubic::{flatten_cubic_bezier, find_cubic_bezier_inflection_points};
-pub use flatten_cubic::Flattened;
-use cubic_to_quadratic::*;
-use monotonic::Monotonic;
-use utils::{min_max, cubic_polynomial_roots};
-use segment::{Segment, FlattenedForEach, approximate_length_from_flattening, BoundingRect};
-use QuadraticBezierSegment;
 
 use std::ops::Range;
+use std::cmp::Ordering::{Less, Equal, Greater};
 
 /// A 2d curve segment defined by four points: the beginning of the segment, two control
 /// points and the end of the segment.
@@ -66,6 +67,55 @@ impl<S: Scalar> CubicBezierSegment<S> {
             self.to.y * t3;
     }
 
+    /// Return the parameter values corresponding to a given x coordinate.
+    /// See also solve_t_for_x for monotonic curves.
+    pub fn solve_t_for_x(&self, x: S) -> ArrayVec<[S; 3]> {
+        if self.is_a_point(S::ZERO)
+            || (self.non_point_is_linear(S::ZERO) && self.from.x == self.to.x)
+        {
+            return ArrayVec::new();
+        }
+
+        self.parameters_for_xy_value(x, self.from.x, self.ctrl1.x, self.ctrl2.x, self.to.x)
+    }
+
+    /// Return the parameter values corresponding to a given y coordinate.
+    /// See also solve_t_for_y for monotonic curves.
+    pub fn solve_t_for_y(&self, y: S) -> ArrayVec<[S; 3]> {
+        if self.is_a_point(S::ZERO)
+            || (self.non_point_is_linear(S::ZERO) && self.from.y == self.to.y)
+        {
+            return ArrayVec::new();
+        }
+
+        self.parameters_for_xy_value(y, self.from.y, self.ctrl1.y, self.ctrl2.y, self.to.y)
+    }
+
+    fn parameters_for_xy_value(
+        &self,
+        value: S,
+        from: S,
+        ctrl1: S,
+        ctrl2: S,
+        to: S,
+    ) -> ArrayVec<[S; 3]> {
+        let mut result = ArrayVec::new();
+
+        let a = -from + S::THREE * ctrl1 - S::THREE * ctrl2 + to;
+        let b = S::THREE * from - S::SIX * ctrl1 + S::THREE * ctrl2;
+        let c = -S::THREE * from + S::THREE * ctrl1;
+        let d = from - value;
+
+        let roots = cubic_polynomial_roots(a, b, c, d);
+        for root in roots {
+            if root > S::ZERO && root < S::ONE {
+                result.push(root);
+            }
+        }
+
+        result
+    }
+
     #[inline]
     fn derivative_coefficients(&self, t: S) -> (S, S, S, S) {
         let t2 = t*t;
@@ -100,12 +150,23 @@ impl<S: Scalar> CubicBezierSegment<S> {
 
     /// Return the sub-curve inside a given range of t.
     ///
-    /// This is equivalent splitting at the range's end points.
+    /// This is equivalent to splitting at the range's end points.
     pub fn split_range(&self, t_range: Range<S>) -> Self {
-        let t1 = t_range.start;
-        let t2 = (t_range.end - t_range.start) / (S::ONE - t_range.start);
+        let (t0, t1) = (t_range.start, t_range.end);
+        let from = self.sample(t0);
+        let to = self.sample(t1);
 
-        self.after_split(t1).before_split(t2)
+        let d = QuadraticBezierSegment {
+            from: (self.ctrl1 - self.from).to_point(),
+            ctrl: (self.ctrl2 - self.ctrl1).to_point(),
+            to: (self.to - self.ctrl2).to_point(),
+        };
+
+        let dt = t1 - t0;
+        let ctrl1 = from + d.sample(t0).to_vector() * dt;
+        let ctrl2 = to - d.sample(t1).to_vector() * dt;
+
+        CubicBezierSegment { from, ctrl1, ctrl2, to }
     }
 
     /// Split this curve into two sub-curves.
@@ -173,34 +234,56 @@ impl<S: Scalar> CubicBezierSegment<S> {
         if (self.from - self.to).square_length() < epsilon {
             return false;
         }
-        let line = self.baseline().to_line().equation();
-        line.distance_to_point(&self.ctrl1) < tolerance
-            && line.distance_to_point(&self.ctrl2) < tolerance
+
+        self.non_point_is_linear(tolerance)
     }
 
-    /// Computes a "fat line" of this segment.
+    #[inline]
+    fn non_point_is_linear(&self, tolerance: S) -> bool {
+        let line = self.baseline().to_line().equation();
+        line.distance_to_point(&self.ctrl1) <= tolerance
+            && line.distance_to_point(&self.ctrl2) <= tolerance
+    }
+
+    pub(crate) fn is_a_point(&self, tolerance: S) -> bool {
+        let tolerance_squared = tolerance * tolerance;
+        // Use <= so that tolerance can be zero.
+        (self.from - self.to).square_length() <= tolerance_squared
+            && (self.from - self.ctrl1).square_length() <= tolerance_squared
+            && (self.to - self.ctrl2).square_length() <= tolerance_squared
+    }
+
+    /// Computes the signed distances (min <= 0 and max >= 0) from the baseline of this
+    /// curve to its two "fat line" boundary lines.
     ///
-    /// A fat line is two convervative lines between which the segment
+    /// A fat line is two conservative lines between which the segment
     /// is fully contained.
-    pub fn fat_line(&self) -> (LineEquation<S>, LineEquation<S>) {
+    pub(crate) fn fat_line_min_max(&self) -> (S, S) {
         let baseline = self.baseline().to_line().equation();
-        let (mut d1, mut d2) = min_max(
+        let (d1, d2) = min_max(
             baseline.signed_distance_to_point(&self.ctrl1),
             baseline.signed_distance_to_point(&self.ctrl2),
         );
 
-        d1 = S::min(d1, S::ZERO);
-        d2 = S::max(d2, S::ZERO);
-
-        let frac_3_4 = S::THREE / S::FOUR;
-
-        if (d1 * d2).is_sign_positive() {
-            d1 = d1 * frac_3_4;
-            d2 = d2 * frac_3_4;
+        let factor = if (d1 * d2) > S::ZERO {
+            S::THREE / S::FOUR
         } else {
-            d1 = d1 * frac_3_4 * frac_3_4;
-            d2 = d2 * frac_3_4 * frac_3_4;
-        }
+            S::FOUR / S::NINE
+        };
+
+        let d_min = factor * S::min(d1, S::ZERO);
+        let d_max = factor * S::max(d2, S::ZERO);
+
+        (d_min, d_max)
+    }
+
+    /// Computes a "fat line" of this segment.
+    ///
+    /// A fat line is two conservative lines between which the segment
+    /// is fully contained.
+    pub fn fat_line(&self) -> (LineEquation<S>, LineEquation<S>) {
+        let baseline = self.baseline().to_line().equation();
+        let (d1, d2) = self.fat_line_min_max();
 
         (baseline.offset(d1), baseline.offset(d2))
     }
@@ -209,10 +292,10 @@ impl<S: Scalar> CubicBezierSegment<S> {
     #[inline]
     pub fn transform(&self, transform: &Transform2D<S>) -> Self {
         CubicBezierSegment {
-            from: transform.transform_point(&self.from),
-            ctrl1: transform.transform_point(&self.ctrl1),
-            ctrl2: transform.transform_point(&self.ctrl2),
-            to: transform.transform_point(&self.to)
+            from: transform.transform_point(self.from),
+            ctrl1: transform.transform_point(self.ctrl1),
+            ctrl2: transform.transform_point(self.ctrl2),
+            to: transform.transform_point(self.to)
         }
     }
 
@@ -550,7 +633,89 @@ impl<S: Scalar> CubicBezierSegment<S> {
         self.is_x_monotonic() && self.is_y_monotonic()
     }
 
-    /// Computes the intersections (if any) between this segment a line.
+    /// Computes the intersections (if any) between this segment and another one.
+    ///
+    /// The result is provided in the form of the `t` parameters of each point along the curves. To
+    /// get the intersection points, sample the curves at the corresponding values.
+    ///
+    /// Returns endpoint intersections where an endpoint intersects the interior of the other curve,
+    /// but not endpoint/endpoint intersections.
+    ///
+    /// Returns no intersections if either curve is a point.
+    pub fn cubic_intersections_t(&self, curve: &CubicBezierSegment<S>) -> ArrayVec<[(S, S); 9]> {
+        cubic_bezier_intersections_t(self, curve)
+    }
+
+    /// Computes the intersection points (if any) between this segment and another one.
+    pub fn cubic_intersections(&self, curve: &CubicBezierSegment<S>) -> ArrayVec<[Point<S>; 9]> {
+        let intersections = self.cubic_intersections_t(curve);
+
+        let mut result_with_repeats = ArrayVec::<[_; 9]>::new();
+        for (t, _) in intersections {
+            result_with_repeats.push(self.sample(t));
+        }
+
+        // We can have up to nine "repeated" values here (for example: two lines, each of which
+        // overlaps itself 3 times, intersecting in their 3-fold overlaps). We make an effort to
+        // dedupe the results, but that's hindered by not having predictable control over how far
+        // the repeated intersections can be from each other (and then by the fact that true
+        // intersections can be arbitrarily close), so the results will never be perfect.
+
+        let pair_cmp = |s: &Point<S>, t: &Point<S>| {
+            if s.x < t.x || (s.x == t.x && s.y < t.y) {
+                Less
+            } else if s.x == t.x && s.y == t.y {
+                Equal
+            } else {
+                Greater
+            }
+        };
+        result_with_repeats.sort_unstable_by(pair_cmp);
+        if result_with_repeats.len() <= 1 {
+            return result_with_repeats;
+        }
+
+        #[inline]
+        fn dist_sq<S: Scalar>(p1: &Point<S>, p2: &Point<S>) -> S {
+            (p1.x - p2.x) * (p1.x - p2.x) + (p1.y - p2.y) * (p1.y - p2.y)
+        }
+
+        let epsilon_squared = S::EPSILON * S::EPSILON;
+        let mut result = ArrayVec::new();
+        let mut reference_intersection = &result_with_repeats[0];
+        result.push(*reference_intersection);
+        for i in 1..result_with_repeats.len() {
+            let intersection = &result_with_repeats[i];
+            if dist_sq(reference_intersection, intersection) < epsilon_squared {
+                continue;
+            } else {
+                result.push(*intersection);
+                reference_intersection = intersection;
+            }
+        }
+
+        result
+    }
+
+    /// Computes the intersections (if any) between this segment a quadratic bézier segment.
+    ///
+    /// The result is provided in the form of the `t` parameters of each point along the curves. To
+    /// get the intersection points, sample the curves at the corresponding values.
+    ///
+    /// Returns endpoint intersections where an endpoint intersects the interior of the other curve,
+    /// but not endpoint/endpoint intersections.
+    ///
+    /// Returns no intersections if either curve is a point.
+    pub fn quadratic_intersections_t(&self, curve: &QuadraticBezierSegment<S>) -> ArrayVec<[(S, S); 9]> {
+        self.cubic_intersections_t(&curve.to_cubic())
+    }
+
+    /// Computes the intersection points (if any) between this segment and a quadratic bézier segment.
+    pub fn quadratic_intersections(&self, curve: &QuadraticBezierSegment<S>) -> ArrayVec<[Point<S>; 9]> {
+        self.cubic_intersections(&curve.to_cubic())
+    }
+
+    /// Computes the intersections (if any) between this segment and a line.
     ///
     /// The result is provided in the form of the `t` parameters of each
     /// point along curve. To get the intersection points, sample the curve
@@ -590,6 +755,7 @@ impl<S: Scalar> CubicBezierSegment<S> {
         return result;
     }
 
+    /// Computes the intersection points (if any) between this segment and a line.
     pub fn line_intersections(&self, line: &Line<S>) -> ArrayVec<[Point<S>; 3]> {
         let intersections = self.line_intersections_t(&line);
 
@@ -601,7 +767,7 @@ impl<S: Scalar> CubicBezierSegment<S> {
         return result;
     }
 
-    /// Computes the intersections (if any) between this segment a line segment.
+    /// Computes the intersections (if any) between this segment and a line segment.
     ///
     /// The result is provided in the form of the `t` parameters of each
     /// point along curve and segment. To get the intersection points, sample
@@ -612,16 +778,29 @@ impl<S: Scalar> CubicBezierSegment<S> {
         }
 
         let intersections = self.line_intersections_t(&segment.to_line());
-        let aabb = segment.bounding_rect();
 
         let mut result = ArrayVec::new();
+        if intersections.len() == 0 {
+            return result;
+        }
+
+        let seg_is_mostly_vertical =
+            S::abs(segment.from.y - segment.to.y) >= S::abs(segment.from.x - segment.to.x);
+        let (seg_long_axis_min, seg_long_axis_max) = if seg_is_mostly_vertical {
+            segment.bounding_range_y()
+        } else {
+            segment.bounding_range_x()
+        };
+
         for t in intersections {
-            if aabb.contains(&self.sample(t)) {
+            let intersection_xy = if seg_is_mostly_vertical { self.y(t) } else { self.x(t) };
+            if intersection_xy >= seg_long_axis_min && intersection_xy <= seg_long_axis_max {
                 let t2 = (self.sample(t) - segment.from).length() / segment.length();
-                result.push((t,t2));
+                result.push((t, t2));
             }
         }
-        return result;
+
+        result
     }
 
     #[inline]
@@ -832,7 +1011,7 @@ fn monotonic_solve_t_for_x() {
 
 #[test]
 fn fat_line() {
-    use math::point;
+    use crate::math::point;
 
     let c1 = CubicBezierSegment {
         from: point(1.0f32, 2.0),
@@ -863,6 +1042,21 @@ fn fat_line() {
         assert!(l1.signed_distance_to_point(&c2.sample(t)) >= -0.000001);
         assert!(l2.signed_distance_to_point(&c2.sample(t)) <= 0.000001);
     }
+
+    let c3 = CubicBezierSegment {
+        from: point(0.0f32, 1.0),
+        ctrl1: point(0.5, 0.0),
+        ctrl2: point(0.5, 0.0),
+        to: point(1.0, 1.0),
+    };
+
+    let (l1, l2) = c3.fat_line();
+
+    for i in 0..100 {
+        let t = i as f32 / 99.0;
+        assert!(l1.signed_distance_to_point(&c3.sample(t)) >= -0.000001);
+        assert!(l2.signed_distance_to_point(&c3.sample(t)) <= 0.000001);
+    }
 }
 
 #[test]
@@ -889,7 +1083,7 @@ fn is_linear() {
 
 #[test]
 fn test_monotonic() {
-    use math::point;
+    use crate::math::point;
     let curve = CubicBezierSegment {
         from: point(1.0, 1.0),
         ctrl1: point(10.0, 2.0),
@@ -902,3 +1096,123 @@ fn test_monotonic() {
         assert!(sub_curve.is_monotonic());
     });
 }
+
+#[test]
+fn test_line_segment_intersections() {
+    use crate::math::point;
+    fn assert_approx_eq(a: ArrayVec<[(f32, f32); 3]>, b: &[(f32, f32)], epsilon: f32) {
+        for i in 0..a.len() {
+            if f32::abs(a[i].0 - b[i].0) > epsilon || f32::abs(a[i].1 - b[i].1) > epsilon {
+                println!("{:?} != {:?}", a, b);
+            }
+            assert!((a[i].0 - b[i].0).abs() <= epsilon && (a[i].1 - b[i].1).abs() <= epsilon);
+        }
+        assert_eq!(a.len(), b.len());
+    }
+
+    let epsilon = 0.0001;
+
+    // Make sure we find intersections with horizontal and vertical lines.
+
+    let curve1 = CubicBezierSegment {
+        from: point(-1.0, -1.0),
+        ctrl1: point(0.0, 4.0),
+        ctrl2: point(10.0, -4.0),
+        to: point(11.0, 1.0),
+    };
+    let seg1 = LineSegment { from: point(0.0, 0.0), to: point(10.0, 0.0) };
+    assert_approx_eq(curve1.line_segment_intersections_t(&seg1), &[(0.5, 0.5)], epsilon);
+
+    let curve2 = CubicBezierSegment {
+        from: point(-1.0, 0.0),
+        ctrl1: point(0.0, 5.0),
+        ctrl2: point(0.0, 5.0),
+        to: point(1.0, 0.0),
+    };
+    let seg2 = LineSegment { from: point(0.0, 0.0), to: point(0.0, 5.0) };
+    assert_approx_eq(curve2.line_segment_intersections_t(&seg2), &[(0.5, 0.75)], epsilon);
+}
+
+#[test]
+fn test_parameters_for_value() {
+    use crate::math::point;
+    fn assert_approx_eq(a: ArrayVec<[f32; 3]>, b: &[f32], epsilon: f32) {
+        for i in 0..a.len() {
+            if f32::abs(a[i] - b[i]) > epsilon {
+                println!("{:?} != {:?}", a, b);
+            }
+            assert!((a[i] - b[i]).abs() <= epsilon);
+        }
+        assert_eq!(a.len(), b.len());
+    }
+
+    {
+        let curve = CubicBezierSegment {
+            from: point(0.0, 0.0),
+            ctrl1: point(0.0, 8.0),
+            ctrl2: point(10.0, 8.0),
+            to: point(10.0, 0.0)
+        };
+
+        let epsilon = 1e-4;
+        assert_approx_eq(curve.solve_t_for_x(5.0), &[0.5], epsilon);
+        assert_approx_eq(curve.solve_t_for_y(6.0), &[0.5], epsilon);
+    }
+    {
+        let curve = CubicBezierSegment {
+            from: point(0.0, 10.0),
+            ctrl1: point(0.0, 10.0),
+            ctrl2: point(10.0, 10.0),
+            to: point(10.0, 10.0)
+        };
+
+        assert_approx_eq(curve.solve_t_for_y(10.0), &[], 0.0);
+    }
+}
+
+#[test]
+fn test_cubic_intersection_deduping() {
+    use crate::math::point;
+
+    let epsilon = 0.0001;
+
+    // Two "line segments" with 3-fold overlaps, intersecting in their overlaps for a total of nine
+    // parameter intersections.
+    let line1 = CubicBezierSegment {
+        from: point(-1_000_000.0, 0.0),
+        ctrl1: point(2_000_000.0, 3_000_000.0),
+        ctrl2: point(-2_000_000.0, -1_000_000.0),
+        to: point(1_000_000.0, 2_000_000.0),
+    };
+    let line2 = CubicBezierSegment {
+        from: point(-1_000_000.0, 2_000_000.0),
+        ctrl1: point(2_000_000.0, -1_000_000.0),
+        ctrl2: point(-2_000_000.0, 3_000_000.0),
+        to: point(1_000_000.0, 0.0),
+    };
+    let intersections = line1.cubic_intersections(&line2);
+    // (If you increase the coordinates above to 10s of millions, you get two returned intersection
+    // points; i.e. the test fails.)
+    assert_eq!(intersections.len(), 1);
+    assert!(f64::abs(intersections[0].x) < epsilon);
+    assert!(f64::abs(intersections[0].y - 1_000_000.0) < epsilon);
+
+    // Two self-intersecting curves that intersect in their self-intersections, for a total of four
+    // parameter intersections.
+    let curve1 = CubicBezierSegment {
+        from: point(-10.0, -13.636363636363636),
+        ctrl1: point(15.0, 11.363636363636363),
+        ctrl2: point(-15.0, 11.363636363636363),
+        to: point(10.0, -13.636363636363636),
+    };
+    let curve2 = CubicBezierSegment {
+        from: point(13.636363636363636, -10.0),
+        ctrl1: point(-11.363636363636363, 15.0),
+        ctrl2: point(-11.363636363636363, -15.0),
+        to: point(13.636363636363636, 10.0),
+    };
+    let intersections = curve1.cubic_intersections(&curve2);
+    assert_eq!(intersections.len(), 1);
+    assert!(f64::abs(intersections[0].x) < epsilon);
+    assert!(f64::abs(intersections[0].y) < epsilon);
+}
