@@ -1,10 +1,10 @@
-use {CubicBezierSegment, Triangle, Line, LineSegment, LineEquation};
-use scalar::Scalar;
-use generic_math::{Point, Vector, Rect, rect, Transform2D};
-use monotonic::Monotonic;
+use crate::{CubicBezierSegment, Triangle, Line, LineSegment, LineEquation};
+use crate::scalar::Scalar;
+use crate::generic_math::{Point, Vector, Rect, rect, Transform2D};
+use crate::monotonic::Monotonic;
+use crate::segment::{Segment, FlatteningStep, FlattenedForEach, BoundingRect};
+use crate::segment;
 use arrayvec::ArrayVec;
-use segment::{Segment, FlatteningStep, FlattenedForEach, BoundingRect};
-use segment;
 
 use std::ops::Range;
 use std::mem;
@@ -164,19 +164,12 @@ impl<S: Scalar> QuadraticBezierSegment<S> {
     ///
     /// This is equivalent splitting at the range's end points.
     pub fn split_range(&self, t_range: Range<S>) -> Self {
-        let t1 = t_range.start;
-        let t2 = t_range.end;
+        let t0 = t_range.start;
+        let t1 = t_range.end;
 
-        debug_assert!(t1 >= S::ZERO);
-        debug_assert!(t2 <= S::ONE);
-        debug_assert!(t1 <= t2);
-        debug_assert!(t1 != S::ONE);
-
-        let from = self.sample(t1);
-        let to = self.sample(t2);
-        let a = self.from.lerp(self.ctrl, t1);
-        let b = self.ctrl.lerp(self.to, t1);
-        let ctrl = a.lerp(b, (t2 - t1) / (S::ONE - t1));
+        let from = self.sample(t0);
+        let to = self.sample(t1);
+        let ctrl = from + (self.ctrl - self.from).lerp(self.to - self.ctrl, t0) * (t1 - t0);
 
         QuadraticBezierSegment { from, ctrl, to }
     }
@@ -254,9 +247,9 @@ impl<S: Scalar> QuadraticBezierSegment<S> {
     #[inline]
     pub fn transform(&self, transform: &Transform2D<S>) -> Self {
         QuadraticBezierSegment {
-            from: transform.transform_point(&self.from),
-            ctrl: transform.transform_point(&self.ctrl),
-            to: transform.transform_point(&self.to)
+            from: transform.transform_point(self.from),
+            ctrl: transform.transform_point(self.ctrl),
+            to: transform.transform_point(self.to)
         }
     }
 
@@ -711,7 +704,7 @@ fn monotonic_solve_t_for_x() {
 
 #[test]
 fn fat_line() {
-    use math::point;
+    use crate::math::point;
 
     let c1 = QuadraticBezierSegment {
         from: point(1.0f32, 2.0),
@@ -750,7 +743,7 @@ fn is_linear() {
 
 #[test]
 fn test_flattening() {
-    use generic_math::point;
+    use crate::generic_math::point;
 
     let c1 = QuadraticBezierSegment {
         from: point(0.0, 0.0),
